
var KPDBHeaders = (function($) {

  /**
  From the KeePass doc:

  [ 4 bytes] DWORD    dwSignature1  = 0x9AA2D903
  [ 4 bytes] DWORD    dwSignature2  = 0xB54BFB65
  [ 4 bytes] DWORD    dwFlags
  [ 4 bytes] DWORD    dwVersion       { Ve.Ve.Mj.Mj:Mn.Mn.Bl.Bl }
  [16 bytes] BYTE{16} aMasterSeed
  [16 bytes] BYTE{16} aEncryptionIV
  [ 4 bytes] DWORD    dwGroups        Number of groups in database
  [ 4 bytes] DWORD    dwEntries       Number of entries in database
  [32 bytes] BYTE{32} aContentsHash   SHA-256 hash value of the plain contents
  [32 bytes] BYTE{32} aMasterSeed2    Used for the dwKeyEncRounds AES
                                      master key transformations
  [ 4 bytes] DWORD    dwKeyEncRounds  See above; number of transformations
 */
  function parse_headers(header_data) {
    var signatures = [0x9AA2D903, 0xB54BFB65];
    var length = 124;
    var format = [
      ['signature1',4,'I'],
      ['signature2',4,'I'],
      ['flags',4,'I'],
      ['version',4,'I'],
      ['master_seed',16,'16s'],
      ['encryption_iv',16,'16s'],
      ['ngroups',4,'I'],
      ['nentries',4,'I'],
      ['contents_hash',32,'32s'],
      ['master_seed2',32,'32s'],
      ['key_enc_rounds',4,'I'],
    ];
    var encryption_flags = [
      ['SHA2', 1],
      ['Rijndael', 2],
      ['AES', 2],
      ['ArcFour', 4],
      ['TwoFish', 8]
    ];

    // Parse headers.
    var index = 0;
    var headers = {};
    for (var i in format) {
      var field = format[i];
      var name = field[0];
      var size = field[1];
      var struct_code = field[2];
      var field_data = header_data.slice(index, index+size);
      var value = Struct.Unpack('<' + struct_code, new Uint8Array(field_data), 0);
      headers[name] = value[0];
      index += size;
    }

    // Verify signatures.
    if (signatures[0] != headers['signature1'] || signatures[1] != headers['signature2']) {
      console.error('Signature mismatch: ' + headers['signature1'] + ", " + headers['signature2']);
      return null;
    } else {
      // Determine enc. type.
      headers['encryption_type'] = null;
      for (var i in encryption_flags) {
        var encflag = encryption_flags[i];
        if (encflag[1] & parseInt(headers['flags'], 10)) {
          headers['encryption_type'] = encflag[0];
        }
      }
      console.debug(headers);
      return headers;
    }
  }

  return {
    'parse_headers': parse_headers
  };
})($);
