
var KPDBUtils = (function($) {
  function data2ab(str) {
    var buf = new ArrayBuffer(str.length);
    var arr = new Uint8Array(buf);
    for (var i = 0, len = str.length; i < len; i++) {
      arr[i] = str.charCodeAt(i);
    }
    return buf;
  }

  function wa2ab(word_array) {
    var buf = new ArrayBuffer(word_array.sigBytes);
    var arr = new Uint8Array(buf);
    var len = word_array.sigBytes;
    for (var i = 0; i < len; i++) {
      arr[i] = ( word_array.words[i >>> 2] >>> ( 24 - (i % 4) * 8 ) ) & 0xff;
    }
    return buf;
  }

  function data2wa(data) {
    return CryptoJS.lib.WordArray.create(new Uint8Array(data2ab(data)));
  }

  function extract_headers(data) {
    return data.slice(0,124);
  }

  function extract_payload(data) {
    return data.slice(124);
  }

  function final_key(master_key, headers) {
    var key = CryptoJS.SHA256(data2wa(master_key));
    var rounds = headers['key_enc_rounds'];
    var master_seed = data2wa(headers['master_seed']);
    var slave_seed = data2wa(headers['master_seed2']);
    while (rounds-- > 0) {
      key = CryptoJS.AES.encrypt(key, slave_seed, {'mode': CryptoJS.mode.ECB, 'padding': CryptoJS.pad.NoPadding}).ciphertext;
    }
    key = CryptoJS.SHA256(key);
    key = CryptoJS.SHA256(master_seed.concat(key));
    return key;
  }

  function decrypt_payload_aes_cbc(final_key, headers, payload_data_buf) {
    var payload_data = CryptoJS.lib.WordArray.create(new Uint8Array(payload_data_buf));
    var iv = data2wa(headers['encryption_iv']);
    
    var aesDecryptor = CryptoJS.algo.AES.createDecryptor(final_key, { iv: iv });
    var payload = aesDecryptor.process(payload_data);
    var payload_arr = new Uint8Array(wa2ab(payload));

    return payload_arr.buffer;
  }

  function decrypt_payload(final_key, headers, payload_data_buf) {
    if (['Rijndael', 'AES'].indexOf(headers['encryption_type']) == -1) {
      console.error('Bad encryption type (not "Rijndael" nor "AES"): ' + headers['encryption_type']);
    } else {
      var payload_buf = decrypt_payload_aes_cbc(final_key, headers, payload_data_buf);
      var payload_arr = new Uint8Array(payload_buf);

      if (payload_arr.length > 2147483446 || (!payload_arr.length && headers['ngroups'])) {
        console.error("Decryption failed. The key is wrong or the file is damaged.");
        return null;
      } else if (data2wa(headers['contents_hash']).toString() != CryptoJS.SHA256(CryptoJS.lib.WordArray.create(payload_arr)).toString()) {
        console.error("Decryption failed. Checksum failed.");
        console.error(data2wa(headers['contents_hash']).toString() + " != " + CryptoJS.SHA256(CryptoJS.lib.WordArray.create(payload_arr)).toString());
        return null;
      } else {
        return payload_arr.buffer;
      }
    }
  }

  return {
    'data2ab': data2ab,
    'wa2ab': wa2ab,
    'data2wa': data2wa,
    'extract_headers': extract_headers,
    'extract_payload': extract_payload,
    'final_key': final_key,
    'decrypt_payload_aes_cbc': decrypt_payload_aes_cbc,
    'decrypt_payload': decrypt_payload
  };
})($);
